from aiosqlite.core import Connection
import aiosqlite
from collections import namedtuple
from .errors import RecordNotFound


__all__ = ("EconomyDB",)


class EconomyDB:
    def __init__(self) -> None:
        self.CollectionTuple = namedtuple("Column", ["id", "wallet", "bank"])

    async def connection(self) -> Connection:
        return await aiosqlite.connect("economy.db")

    async def sqlexec(self, query: str, entities=None) -> None:
        async with (con := await self.connection()).cursor() as cur:
            if not entities:
                await cur.execute(query)
            else:
                await cur.execute(query, entities)
            await con.commit()
            await con.close()

    @classmethod
    async def create(cls) -> "EconomyDB":
        await (cl := cls()).sqlexec(
            "CREATE TABLE IF NOT EXISTS users(id integer PRIMARY KEY, wallet INTEGER, bank INTEGER)"
        )
        return cl

    async def read_data(self, user_id: int) -> tuple[int, ...]:
        async with (con := await self.connection()).cursor() as cur:
            if result := (
                await (
                    await cur.execute(
                        "SELECT id, wallet, bank FROM users WHERE id = ?", (user_id,)
                    )
                ).fetchone()
            ):
                return self.CollectionTuple(result[0], result[1], result[2])

    async def create_account(
        self, user_id: int, wallet_amount: int, bank_amount: int
    ) -> tuple[int, ...]:
        res = await self.read_data(user_id)
        if not (res := await self.read_data(user_id)):
            await self.sqlexec(
                "INSERT INTO users(id, wallet, bank) VALUES(?, ?, ?)",
                (
                    user_id,
                    wallet_amount,
                    bank_amount,
                ),
            )
            return self.CollectionTuple(user_id, wallet_amount, bank_amount)
        return self.CollectionTuple(res[0], res[1], res[2])

    async def update_account(
        self, user_id: int, *, wallet_amount: int = None, bank_amount: int = None
    ) -> bool:
        if not (user := await self.read_data(user_id)):
            raise RecordNotFound
        if wallet_amount and not bank_amount:
            await self.sqlexec(
                "UPDATE users SET wallet = ? WHERE id = ?",
                (
                    wallet_amount,
                    user_id,
                ),
            )
            return True
        elif bank_amount and not wallet_amount:
            await self.sqlexec(
                "UPDATE users SET bank = ? WHERE id = ?",
                (
                    bank_amount,
                    user_id,
                ),
            )
            return True
        elif wallet_amount and bank_amount:
            await self.sqlexec(
                "UPDATE users SET wallet = ? , bank = ? WHERE id = ?",
                (
                    wallet_amount,
                    bank_amount,
                    user_id,
                ),
            )
            return True
        return False
